/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _GNOTIFY_H
#define _GNOTIFY_H

#include <gtk/gtk.h>
#include <pthread.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <syslog.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/utsname.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <signal.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>

#define VERSION "1.1"
#define GNOTIFY_ICON "/usr/share/pixmaps/gnome-main-menu.png"
#define GNOTIFY_ETC	"/etc/gnotify.xml"

/* The config-struct. Here all information gnotify got from the 
 * ~/.gnotify.xml will be stored */
struct CONF
{
  int notify_x;
  int notify_y;
  int notify_delay;
  int notify_play;
  int notify_ipdisallows;
	int notify_allowmove;
	int notify_allowhtml;
	int notify_allowreload;
	int notify_allowconfchange;
	int notify_saveconfonexit;
  char notify_ssink[12];
	char notify_logout[256];
  char notify_allowip[128][32];
	
	int notify_verbose;
}cfg;

 /* The Client-Database. Every client gehts one array of this struct
	* and will use it for storing things like his IP, his Socket-
	* Filedescriptor and other stuff. */
struct CLIENTSDB
{
	pthread_t th;
	int sockfd;
	int move_x;
	int move_y;
	int ckboxpress;
	int btnpress;
	int occupied;
	char ip[16];
  char app[128];
  char icon[512];
  char snd[512];
  char msg[512];
  char button[128];
	char ckbox[128];
}cdb[128];

struct SYSINFODB
{
  char k_release[32];
  char k_version[64];
  char machine[16];
  char opsys[32];
  char nodename[64];
}sysinfo;

pthread_mutex_t mutex; /* This is the mutex which is used for a thread-
												* synchronization. */
pthread_t gtkf;
pthread_t NULLTHREAD;
pid_t GNOTIFYPID;

int gnotify_constructor(void);
void gnotify_main(void);

void gnotify_about(void);

/* Configuration-GUI stuff */
void gnotify_cfggui(void);

void ckPlaysound_callback(GtkObject *widget, gpointer user_data);
void ckAllowhtml_callback(GtkObject *widget, gpointer user_data);
void ckAllowcfgchange_callback(GtkObject *widget, gpointer user_data);
void ckSaveconf_callback(GtkObject *widget, gpointer user_data);
void ckAllowmove_callback(GtkObject *widget, gpointer user_data);
void ckAllowreload_callback(GtkObject *widget, gpointer user_data);
void cboSsink_callback(GtkObject *widget, gpointer user_data);
void cboLogout_callback(GtkObject *widget, gpointer user_data);
void cmdAddip_callback(GtkObject *widget, gpointer user_data);
void cmdSave_callback(GtkObject *widget, gpointer user_data);

void gnotify_cfggui_iplist(void);
char *list_to_plain(void);
void cmdAddip_callback(GtkObject *widget, gpointer user_data);
void cmdRemoveip_callback(GtkObject *widget, gpointer user_data);
void cmdSaveip_callback(GtkObject *widget, gpointer user_data);
int searchip(char *ip);
int add_ip_to_list(char *ip);
int remove_ip_from_list(char *ip);
/* End of the cfggui stuff */

void gnotify_atexit(void);

/* This is the gnotify-signal-handler.
 * Parameters:
 * sign - The signal which arrives
 */
void gnotify_signal(int sign);

/* The gnotify-clienthandling function,
 * Parameters:
 * cid - The client-id (needed for accessing the cdb[])
 */
int gnotify_client(int cid);

int gnotify_notify(int cid);
void gnotify_notify_quit(GtkWidget *widget,gpointer data);
void buttonclick(GtkWidget *widget,gpointer data);
void ckboxonclick(GtkWidget *widget,gpointer data);
int playsound(const char *file);

void showmessage(int type, int buttons, char *text);

/* This function gets a free Client-ID from the cdb-struct and returns it */
int cid_getfree(void);

void cid_zombiekill(void);

void getsysinfo(void);

int is_ip(char ip[16]);
char *reverse_strnget(const char* string1, int n);
int printflog(int syslogpriority, const char *text, ...);

int saveconfig(char *config);
int readconfig(char *config);
int createconfig(char *config);

#endif
